import * as i0 from '@angular/core';
import { inject, InjectionToken, Inject, Optional, Injectable, effect, untracked, makeEnvironmentProviders, provideAppInitializer } from '@angular/core';
import { Router } from '@angular/router';
import { NgSsmFeatureState, Store, Logger, provideReducer } from 'ngssm-store';
import { __decorate } from 'tslib';
import update from 'immutability-helper';

const selectNavigationState = (state) => state[NavigationStateSpecification.featureStateKey];
const updateNavigationState = (state, command) => update(state, {
    [NavigationStateSpecification.featureStateKey]: command
});
let NavigationStateSpecification = class NavigationStateSpecification {
    static { this.featureStateKey = 'navigation-state'; }
    static { this.initialState = {
        navigationLocked: false
    }; }
};
NavigationStateSpecification = __decorate([
    NgSsmFeatureState({
        featureStateKey: NavigationStateSpecification.featureStateKey,
        initialState: NavigationStateSpecification.initialState
    })
], NavigationStateSpecification);

const isNavigationLocked = () => {
    const state = inject(Store).state();
    return selectNavigationState(state).navigationLocked;
};
const isNavigationUnLocked = () => {
    const state = inject(Store).state();
    return !selectNavigationState(state).navigationLocked;
};

var NavigationActionType;
(function (NavigationActionType) {
    NavigationActionType["lockNavigation"] = "[NavigationActionType] lockNavigation";
    NavigationActionType["unLockNavigation"] = "[NavigationActionType] unLockNavigation";
})(NavigationActionType || (NavigationActionType = {}));

const NGSSM_NAVIGATION_LOCKING_CONFIG = new InjectionToken('NGSSM_NAVIGATION_LOCKING_CONFIG');

class NavigationReducer {
    constructor(configs) {
        this.processedActions = [];
        this.lockingActions = new Set([NavigationActionType.lockNavigation]);
        this.unlockingActions = new Set([NavigationActionType.unLockNavigation]);
        (configs ?? []).forEach((config) => {
            (config.actionsLockingNavigation ?? []).forEach((command) => this.lockingActions.add(command));
            (config.actionsUnLockingNavigation ?? []).forEach((command) => this.unlockingActions.add(command));
        });
        this.processedActions = [...this.lockingActions, ...this.unlockingActions];
    }
    updateState(state, action) {
        if (this.lockingActions.has(action.type)) {
            return updateNavigationState(state, {
                navigationLocked: { $set: true }
            });
        }
        if (this.unlockingActions.has(action.type)) {
            return updateNavigationState(state, {
                navigationLocked: { $set: false }
            });
        }
        return state;
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: NavigationReducer, deps: [{ token: NGSSM_NAVIGATION_LOCKING_CONFIG, optional: true }], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: NavigationReducer }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "20.0.0", ngImport: i0, type: NavigationReducer, decorators: [{
            type: Injectable
        }], ctorParameters: () => [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [NGSSM_NAVIGATION_LOCKING_CONFIG]
                }, {
                    type: Optional
                }] }] });

const routingEffectInitializer = async () => {
    const store = inject(Store);
    const router = inject(Router);
    const logger = inject(Logger);
    effect(() => {
        const action = store.processedAction();
        const routingAction = action;
        if (routingAction.navigate) {
            logger.information(`[Routing Effect] Processing action ${action.type}`);
            try {
                untracked(() => routingAction.navigate?.(store.state(), router));
                logger.information(`[Routing Effect] Successfully navigated for action ${action.type}`);
            }
            catch (error) {
                logger.error(`[Routing Effect] Error while processing action ${action.type}: ${error}`);
            }
        }
    });
    return true;
};

const initializeNavigation = () => {
    inject(Router).config.forEach((route) => {
        route.canDeactivate = [isNavigationUnLocked, ...(route.canDeactivate ?? [])];
    });
};
const provideNgssmNavigation = () => {
    return makeEnvironmentProviders([
        provideReducer(NavigationReducer),
        provideAppInitializer(initializeNavigation),
        provideAppInitializer(routingEffectInitializer)
    ]);
};

/*
 * Public API Surface of ngssm-navigation
 */

/**
 * Generated bundle index. Do not edit.
 */

export { NGSSM_NAVIGATION_LOCKING_CONFIG, NavigationActionType, NavigationStateSpecification, isNavigationLocked, isNavigationUnLocked, provideNgssmNavigation, routingEffectInitializer, selectNavigationState, updateNavigationState };
//# sourceMappingURL=ngssm-navigation.mjs.map
