import { EnvironmentProviders, InjectionToken } from '@angular/core';
import { Spec } from 'immutability-helper';
import { State } from 'ngssm-store';
import { Router } from '@angular/router';

declare const provideNgssmNavigation: () => EnvironmentProviders;

declare const selectNavigationState: (state: State) => NavigationState;
declare const updateNavigationState: (state: State, command: Spec<NavigationState, never>) => State;
interface NavigationState {
    navigationLocked: boolean;
}
declare class NavigationStateSpecification {
    static readonly featureStateKey = "navigation-state";
    static readonly initialState: NavigationState;
}

declare const isNavigationLocked: () => boolean;
declare const isNavigationUnLocked: () => boolean;

declare enum NavigationActionType {
    lockNavigation = "[NavigationActionType] lockNavigation",
    unLockNavigation = "[NavigationActionType] unLockNavigation"
}

interface NavigationLockingConfig {
    actionsLockingNavigation?: string[];
    actionsUnLockingNavigation?: string[];
}
declare const NGSSM_NAVIGATION_LOCKING_CONFIG: InjectionToken<NavigationLockingConfig>;

interface RoutingAction {
    navigate?(state: State, router: Router): void;
}

declare const routingEffectInitializer: () => Promise<boolean>;

export { NGSSM_NAVIGATION_LOCKING_CONFIG, NavigationActionType, NavigationStateSpecification, isNavigationLocked, isNavigationUnLocked, provideNgssmNavigation, routingEffectInitializer, selectNavigationState, updateNavigationState };
export type { NavigationLockingConfig, NavigationState, RoutingAction };
